######################################################################
#
# The Apache Software License, Version 1.1
# 
# Copyright (c) 1999-2000 The Apache Software Foundation.  All rights 
# reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer. 
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. The end-user documentation included with the redistribution,
#    if any, must include the following acknowledgment:  
#       "This product includes software developed by the
#        Apache Software Foundation (http://www.apache.org/)."
#    Alternately, this acknowledgment may appear in the software itself,
#    if and wherever such third-party acknowledgments normally appear.
# 
# 4. The names "Xerces" and "Apache Software Foundation" must
#    not be used to endorse or promote products derived from this
#    software without prior written permission. For written 
#    permission, please contact apache\@apache.org.
# 
# 5. Products derived from this software may not be called "Apache",
#    nor may "Apache" appear in their name, without prior written
#    permission of the Apache Software Foundation.
# 
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
# ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
# USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
# OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
# ====================================================================
# 
# This software consists of voluntary contributions made by many
# individuals on behalf of the Apache Software Foundation, and was
# originally based on software copyright (c) 1999, International
# Business Machines, Inc., http://www.ibm.com .  For more information
# on the Apache Software Foundation, please see
# <http://www.apache.org/>.
#
######################################################################
#
# SAXCount
#
# This sample is modeled after its Xerces-C counterpart.  You give it an
# XML file and it parses it with a SAX parser and counts what it sees.
#
######################################################################

use strict;
use XML::Xerces;
use Getopt::Std;
use vars qw($opt_v $opt_n);
use Benchmark;
#
# Read and validate command line args
#

my $USAGE = "USAGE: $0 [-v][-n] file\n";
my $VERSION = q[$Id: SAXCount.pl,v 1.1 2001/04/13 22:23:51 jasons Exp $ ];

getopts ('vn') && ($#ARGV == 0)	or die "$USAGE";
-f $ARGV[0] or die "File '$ARGV[0]' does not exist!\n";

my $validate = $opt_v || 0;
my $namespace = $opt_n || 0;
my $file = $ARGV[0];

#
# Count the nodes
#

my $parser = XML::Xerces::SAXParser->new();
$parser->setDoValidation ($validate);
$parser->setDoNamespaces ($namespace);
my $error_handler = new XML::Xerces::perlErrorHandler;
$error_handler->setWarningFunction(\&XML::Xerces::warning);
$error_handler->setErrorFunction(\&XML::Xerces::error);
$error_handler->setFatalErrorFunction(\&XML::Xerces::fatal_error);
$parser->setErrorHandler($error_handler);

my $element_count = 0;
my $ws_count = 0;
my $attr_count = 0;
my $char_count = 0;
my $DOCUMENT_HANDLER = XML::Xerces::perlDocumentHandler->new();
$DOCUMENT_HANDLER->setStartElementFunction(sub{$element_count++;});
$DOCUMENT_HANDLER->setIgnorableWhitespaceFunction(sub{my ($str,$len) = @_;
						      $ws_count += $len;});
$DOCUMENT_HANDLER->setCharactersFunction(sub{my ($str,$len) = @_;
					     $char_count += $len;});
$parser->setDocumentHandler($DOCUMENT_HANDLER);

my $t0 = new Benchmark;
$parser->parse (XML::Xerces::LocalFileInputSource->new($file));
my $t1 = new Benchmark;
my $td = timediff($t1, $t0);

print "$file: duration: ", timestr($td), "\n";
print "elems: $element_count\n"; 
print "attrs: $attr_count\n";
print "whitespace: $ws_count\n";
print "characters: $char_count\n";


__END__
